/****************************************************************************
 ****************************************************************************/

#include "jwm.h"

/****************************************************************************
 ****************************************************************************/
BorderActionType GetBorderActionType(ClientNode *np, int x, int y) {
	BorderActionType action;
	int north;
	int offset;

	if(!(np->borderFlags & BORDER_OUTLINE)) {
		Debug("GetBorderActionType: this shouldn't happen");
		return BA_NONE;
	}

	if(np->borderFlags & BORDER_TITLE) {

		if(y > BORDER_SOUTH && y <= BORDER_SOUTH + BUTTON_HEIGHT) {
			offset = np->width + BORDER_WEST - BUTTON_WIDTH;
			if(np->borderFlags & BORDER_CLOSE) {
				if(x > offset && x < offset + BUTTON_WIDTH) {
					return BA_CLOSE;
				}
				offset -= BUTTON_WIDTH + BUTTON_SPACING;
			}
			if(np->borderFlags & BORDER_MAX) {
				if(x > offset && x < offset + BUTTON_WIDTH) {
					return BA_MAXIMIZE;
				}
				offset -= BUTTON_WIDTH + BUTTON_SPACING;
			}
			if(np->borderFlags & BORDER_MIN) {
				if(x > offset && x < offset + BUTTON_WIDTH) {
					return BA_MINIMIZE;
				}
			}
			return BA_MOVE;
		}

		north = BORDER_NORTH;
	} else {
		north = BORDER_SOUTH;
	}

	action = BA_NONE;
	if(x < BORDER_WEST) {
		action |= BA_RESIZE_W | BA_RESIZE;
	}
	if(x > (int)np->width) {
		action |= BA_RESIZE_E | BA_RESIZE;
	}
	if(y < BORDER_SOUTH) {
		action |= BA_RESIZE_N | BA_RESIZE;
	}
	if(y - north + BORDER_SOUTH > (int)np->height) {
		action |= BA_RESIZE_S | BA_RESIZE;
	}
	return action;

}

/****************************************************************************
 ****************************************************************************/
void DrawBorder(ClientNode *np) {
	unsigned int width, height;
	long borderPixel, borderTextPixel;
	Pixmap canvas;
	GC canvasGC;

	if(shouldExit || !np->isMapped) {
		return;
	}

	width = np->width + BORDER_EAST + BORDER_WEST;
	if(np->borderFlags & BORDER_TITLE) {
		height = np->height + BORDER_NORTH + BORDER_SOUTH;
	} else {
		height = np->height + 2 * BORDER_SOUTH;
	}

	if(np->isActive) {
		borderPixel = activeBorderColor.pixel;
		borderTextPixel = activeBorderTextColor.pixel;
	} else {
		borderPixel = borderColor.pixel;
		borderTextPixel = borderTextColor.pixel;
	}

	canvas = XCreatePixmap(display, np->parent, width, height,
		DefaultDepth(display, 0));
	canvasGC = XCreateGC(display, canvas, 0, NULL);

	XSetForeground(display, canvasGC, borderPixel);
	XFillRectangle(display, canvas, canvasGC, 1, 1,
		width - 2, height - 2);

	if(np->name && np->name[0] && (np->borderFlags & BORDER_TITLE)) {
		XSetForeground(display, canvasGC, borderTextPixel);
		XSetBackground(display, canvasGC, borderPixel);
		XSetFont(display, canvasGC, borderFont->fid);
		XDrawImageString(display, canvas, canvasGC,
			BORDER_WEST, 18, np->name, strlen(np->name));

	}

	DrawBorderButtons(np, canvas, canvasGC);

	XSetForeground(display, canvasGC, borderOutlineColor.pixel);
	XDrawRectangle(display, canvas, canvasGC, 0, 0, width - 1, height - 1);

	XCopyArea(display, canvas, np->parent, np->parentGC, 0, 0,
		width, height, 0, 0);

	XFreeGC(display, canvasGC);
	XFreePixmap(display, canvas);

}

/****************************************************************************
 ****************************************************************************/
void DrawBorderButtons(ClientNode *np, Drawable canvas, GC gc) {
	XImage *image;
	BorderActionType action;
	unsigned int offset;
	int x, y;

	if(!(np->borderFlags & BORDER_TITLE)) {
		return;
	}

	GetMousePosition(&x, &y);
	if(np->borderFlags & BORDER_OUTLINE) {
		x = x - np->x + BORDER_WEST;
	} else {
		x = x - np->x;
	}
	y = y - np->y + BORDER_NORTH;
	action = GetBorderActionType(np, x, y);

	offset = np->width + BORDER_WEST - 16;

	if(np->borderFlags & BORDER_CLOSE) {
		if(action == BA_CLOSE) {
			image = closeOverImage;
		} else {
			image = closeImage;
		}
		XPutImage(display, canvas, gc, image, 0, 0,
			offset, BORDER_NORTH / 2 - image->height / 2,
			image->width, image->height);
		offset -= BUTTON_WIDTH + BUTTON_SPACING;
	}

	if(np->borderFlags & BORDER_MAX) {
		if(action == BA_MAXIMIZE) {
			image = maxOverImage;
		} else {
			image = maxImage;
		}
		XPutImage(display, canvas, gc, image, 0, 0,
			offset, BORDER_NORTH / 2 - image->height / 2,
			image->width, image->height);
		offset -= BUTTON_WIDTH + BUTTON_SPACING;
	}

	if(np->borderFlags & BORDER_MIN) {
		if(action == BA_MINIMIZE) {
			image = minOverImage;
		} else {
			image = minImage;
		}
		XPutImage(display, canvas, gc, image, 0, 0,
			offset, BORDER_NORTH / 2 - image->height / 2,
			image->width, image->height);
	}

}

